@@ -8,48 +8,52 @@ import os
 from functools import wraps
 from subprocess import Popen, DEVNULL
 from tmc.errors import (TMCError, NoCourseSelected, NoExerciseSelected,
-                        NoSuchCourse, NoSuchExercise)
+                        NoSuchCourse, NoSuchExercise, APIError)
 from tmc.prompt import yn_prompt, custom_prompt
 from tmc.spinner import SpinnerDecorator
-from tmc import db, api, files, menu, VERSION
+from tmc import db, api, files, menu
 from tmc.models import Course, Exercise, Config
 
 import peewee
 
 
-def needs_a_course(func):
+def selected_course(func):
     @wraps(func)
     def inner(*args, **kwargs):
-        Course.get_selected()
-        return func(*args, **kwargs)
+        course = Course.get_selected()
+        return func(course, *args, **kwargs)
     return inner
 
 
-def wrap_tmc(func):
+def selected_exercise(func):
     @wraps(func)
     def inner(*args, **kwargs):
-        try:
-            ret = func(*args, **kwargs)
-        except TMCError as e:
-            print(e)
-            exit(-1)
-        return ret
+        exercise = Exercise.get_selected()
+        return func(exercise, *args, **kwargs)
     return inner
 
 
 @aliases("reset")
-@wrap_tmc
 def resetdb():
+    """
+    Resets the local database.
+    """
+    print("This won't remove any of your files,",
+          "but instead the local database that tracks your progress.")
     if yn_prompt("Reset database", False):
         db.reset()
+        print("Database resetted. You will need to tmc configure again.")
 
 
 @aliases("up")
-@wrap_tmc
 @arg("-c", "--course", action="store_true", help="Update courses instead.")
 def update(course=False):
+    """
+    Update the metadata of courses and or exercises from server.
+    """
     if course:
-        @SpinnerDecorator("Done.")
+        @SpinnerDecorator("Updated course metadata.",
+                          waitmsg="Updating course metadata.")
         def update_course():
             for course in api.get_courses():
                 old = None
@@ -65,7 +69,8 @@ def update(course=False):
     else:
         selected = Course.get_selected()
 
-        @SpinnerDecorator()
+        @SpinnerDecorator("Updated exercise metadata.",
+                          waitmsg="Updating exercise metadata.")
         def update_exercise():
             for exercise in api.get_exercises(selected.tid):
                 old = None
@@ -78,44 +83,60 @@ def update(course=False):
                     old.course = selected.id
                     old.is_attempted = exercise["attempted"]
                     old.is_completed = exercise["completed"]
-                    old.deadline = exercise.get("deadline", None)
+                    old.deadline = exercise.get("deadline")
+                    old.is_downloaded = os.path.isdir(old.path())
                     old.save()
                 else:
-                    Exercise.create(tid=exercise["id"],
-                                    name=exercise["name"],
-                                    course=selected.id,
-                                    is_attempted=exercise["attempted"],
-                                    is_completed=exercise["completed"],
-                                    deadline=exercise.get("deadline", None))
+                    ex = Exercise.create(tid=exercise["id"],
+                                         name=exercise["name"],
+                                         course=selected.id,
+                                         is_attempted=exercise["attempted"],
+                                         is_completed=exercise["completed"],
+                                         deadline=exercise.get("deadline"))
+                    ex.is_downloaded = os.path.isdir(ex.path())
+                    ex.save()
         update_exercise()
-        print("Done.")
 
 
-@arg("-f", "--force", default=False,
-     action="store_true", help="Should the download be forced.")
 @aliases("dl")
-@needs_a_course
-@wrap_tmc
-def download(what="all", force=False):
-    what = what.upper()
-    selected = Course.get_selected()
-    if what == "ALL":
-        for exercise in selected.exercises:
-            files.download_file(exercise.tid, force=force)
-    elif what == "REMAINING":
-        for exercise in selected.exercises:
-            if not exercise.is_completed:
-                files.download_file(exercise.tid, force=force)
+@arg("-i", "--id", help="Download this ID.")
+@arg("-a", "--all", default=False, action="store_true",
+     help="Download all exercises.")
+@arg("-f", "--force", default=False, action="store_true",
+     help="Should the download be forced.")
+@arg("-u", "--upgrade", default=False, action="store_true",
+     help="Should the Java target be upgraded from 1.6 to 1.7")
+@selected_course
+def download(course, id=None, all=False, force=False, upgrade=False):
+    """
+    Download the exercises from the server.
+    """
+
+    def dl(id):
+        files.download_file(id, force=force, update_java=upgrade)
+
+    if all:
+        for exercise in list(course.exercises):
+            dl(exercise.tid)
+    elif id is not None:
+        dl(int(id))
     else:
-        files.download_file(int(what), force=force)
+        for exercise in list(course.exercises):
+            if not exercise.is_completed:
+                dl(exercise.tid)
+            else:
+                exercise.update_downloaded()
 
 
 @aliases("te")
-@needs_a_course
-@wrap_tmc
-def test(what=None):
-    if what is not None:
-        if not files.test(int(what)):
+@arg("-i", "--id", help="Test this ID.")
+@selected_course
+def test(course, id=None):
+    """
+    Run tests on the selected exercise.
+    """
+    if id is not None:
+        if not files.test(int(id)):
             exit(-1)
     else:
         sel = Exercise.get_selected()
@@ -124,16 +145,19 @@ def test(what=None):
         files.test(sel.tid)
 
 
+@aliases("su")
+@arg("-i", "--id", help="Submit this ID.")
 @arg("-p", "--pastebin", default=False, action="store_true",
      help="Should the submission be sent to TMC pastebin.")
 @arg("-r", "--review", default=False, action="store_true",
      help="Request a review for this submission.")
-@aliases("su")
-@needs_a_course
-@wrap_tmc
-def submit(what=None, pastebin=False, review=False):
-    if what is not None:
-        files.submit(int(what), pastebin=pastebin, request_review=review)
+@selected_course
+def submit(course, id=None, pastebin=False, review=False):
+    """
+    Submit the selected exercise to the server.
+    """
+    if id is not None:
+        files.submit(int(id), pastebin=pastebin, request_review=review)
     else:
         sel = Exercise.get_selected()
         if not sel:
@@ -142,10 +166,12 @@ def submit(what=None, pastebin=False, review=False):
 
 
 @aliases("sel")
-@wrap_tmc
 @arg("-c", "--course", action="store_true", help="Select a course instead.")
 @arg("-i", "--id", help="Select this ID without invoking the curses UI.")
 def select(course=False, id=None):
+    """
+    Select a course or an exercise.
+    """
     if course:
         update(course=True)
         og = None
@@ -169,7 +195,7 @@ def select(course=False, id=None):
             sel.set_select()
             update()
             if sel.path == "":
-                selpath()
+                select_a_path()
             oldex = None
             try:
                 oldex = Exercise.get_selected()
@@ -195,7 +221,7 @@ def select(course=False, id=None):
         sel = Course.get_selected()
         ret = id
         if not ret:
-            ret = menu.launch("Select a exercise",
+            ret = menu.launch("Select an exercise",
                               Exercise.select().where(
                                   Exercise.course == sel.id).execute(),
                               start_index)
@@ -210,21 +236,25 @@ def select(course=False, id=None):
 
 
 @aliases("skip")
-@needs_a_course
-@wrap_tmc
-def next():
+@selected_course
+def next(course, num=1):
+    """
+    Go to the next exercise.
+    """
     sel = None
     try:
         sel = Exercise.get_selected()
     except NoExerciseSelected:
         pass
-    exercises = Course.get_selected().exercises
     try:
         if sel is None:
-            sel = [i for i in exercises][0]
+            for i in course.exercises:
+                if i.is_downloaded:
+                    sel = i
+                    break
         else:
             try:
-                sel = Exercise.get(Exercise.id == sel.id + 1)
+                sel = Exercise.get(Exercise.id == sel.id + num)
             except peewee.DoesNotExist:
                 print("There are no more exercises in this course.")
                 exit(-1)
@@ -236,49 +266,61 @@ def next():
         # unsupported type.
         # This might be a bug in peewee.
     sel.set_select()
-    print("Selected {}".format(sel))
+    listall(single=sel)
+    # print("Selected {}".format(sel))
+
+
+@aliases("prev")
+def previous():
+    next(num=-1)
 
 
 @aliases("ls")
-@wrap_tmc
-@needs_a_course
-def listall():
-    exercises = Course.get_selected().exercises
+@selected_course
+def listall(course, single=None):
+    """
+    Lists all of the exercises in the current course.
+    """
+
+    def bs(val):
+        return "●" if val else " "
+
+    def bc(val):
+        return "\033[32m✔\033[0m" if val else "\033[31m✘\033[0m"
+
+    def format(exercise):
+        return "{0} │ {1} │ {2} │ {3} │ {4}".format(exercise.tid,
+                                                    bs(exercise.is_selected),
+                                                    bc(exercise.is_downloaded),
+                                                    bc(exercise.is_completed),
+                                                    exercise.menuname())
+
     print("ID{0}│ {1} │ {2} │ {3} │ {4}".format(
-        (len(str(exercises[0].tid)) - 1) * " ",
+        (len(str(course.exercises[0].tid)) - 1) * " ",
         "S", "D", "C", "Name"
     ))
-    for exercise in exercises:
+    if single:
+        print(format(single))
+        return
+    for exercise in course.exercises:
         # ToDo: use a pager
-        print("{0} │ {1} │ {2} │ {3} │ {4}".format(exercise.tid,
-                                                   bts(exercise.is_selected),
-                                                   btc(exercise.is_downloaded),
-                                                   btc(exercise.is_completed),
-                                                   exercise.name))
+        print(format(exercise))
 
 
-def bts(val):
-    return "●" if val else " "
-
-
-def btc(val):
-    return "\033[32m✔\033[0m" if val else "\033[31m✘\033[0m"
-
-
-@needs_a_course
 @arg('command', help='The command')
-@wrap_tmc
-def run(command):
+@selected_exercise
+def run(exercise, command):
     """
     Spawns a process with `command path-of-exercise`
     """
-    exercise = Exercise.get_selected()
     Popen(['nohup', command, exercise.path()], stdout=DEVNULL, stderr=DEVNULL)
 
 
-@aliases("init")
-@aliases("conf")
+@aliases("init", "conf")
 def configure():
+    """
+    Configure tmc.py to use your account.
+    """
     if Config.has():
         sure = input("Override old configuration [y/N]: ")
         if sure.upper() != "Y":
@@ -287,16 +329,18 @@ def configure():
     server = input("Server url [https://tmc.mooc.fi/mooc/]: ")
     if len(server) == 0:
         server = "https://tmc.mooc.fi/mooc/"
-    while(True):
+    while True:
         username = input("Username: ")
         password = getpass.getpass("Password: ")
         # wow, such security
         token = base64.b64encode(bytes("{0}:{1}".format(username, password),
                                        'utf-8')).decode("utf-8")
+
+        api.configure(server, token)
         try:
-            api.configure(server, token)
-        except Exception as e:  # ToDo: Better exception
-            raise e
+            api.make_request("courses.json")
+        except APIError as e:
+            print(e)
             if yn_prompt("Retry authentication"):
                 continue
             exit()
@@ -304,17 +348,16 @@ def configure():
     select(course=True)
 
 
-@needs_a_course
-def selpath():
-    sel = Course.get_selected()
+@selected_course
+def select_a_path(course):
     defpath = os.path.join(os.path.expanduser("~"),
                            "tmc",
-                           sel.name)
+                           course.name)
     path = input("File download path [{0}]: ".format(defpath))
     if len(path) == 0:
         path = defpath
-    sel.path = path
-    sel.save()
+    course.path = path
+    course.save()
     ret = custom_prompt("Download exercises R: Remaining A: All N: None",
                         ["r", "a", "n"],
                         "r")
@@ -323,19 +366,27 @@ def selpath():
     elif ret == "r":
         download("remaining")
     else:
-        print("You can download the exercises with `tmc download all`")
+        print("You can download the exercises with `tmc download --all`")
 
 
 def version():
-    print("tmc.py version {0}".format(VERSION))
+    """
+    Prints the version and exits.
+    """
+    from tmc import version
+    print("tmc.py version {0}".format(version))
     print("Copyright 2014 Juhani Imberg")
 
 
 def main():
     parser = argh.ArghParser()
-    parser.add_commands([select, update, download, test, submit, next, resetdb,
-                         configure, version, listall, run])
-    parser.dispatch()
+    parser.add_commands([select, update, download, test, submit, next,
+                         previous, resetdb, configure, version, listall, run])
+    try:
+        parser.dispatch()
+    except TMCError as e:
+        print(e)
+        exit(-1)
 
 
 if __name__ == "__main__":
