@@ -13,6 +13,7 @@ class API:
         self.server_url = ""
         self.auth_header = ""
         self.configured = False
+        self.tried_configuration = False
         self.api_version = 7
         # uncomment client and client_version after tmc.mooc.fi/mooc upgrades
         """ self.params = {
@@ -24,32 +25,29 @@ class API:
             "api_version": self.api_version
         }
 
-        self.db_configure()
-
     def db_configure(self):
-        try:
-            url = Config.get_value("url")
-            token = Config.get_value("token")
-            self.configure(url, token)
-        except Exception:
-            pass  # print(e)
+        url = Config.get_value("url")
+        token = Config.get_value("token")
+        self.configure(url, token)
 
     def configure(self, url, token):
         self.server_url = url
         self.auth_header = {"Authorization": "Basic {0}".format(token)}
         self.configured = True
-
-        self.make_request("courses.json")
+        self.tried_configuration = True
 
         Config.set("url", url)
         Config.set("token", token)
 
-    def make_request(self, slug):
+    def make_request(self, slug, timeout=10):
+        if not self.tried_configuration:
+            self.db_configure()
         if not self.configured:
             raise APIError("API needs to be configured before use!")
         req = requests.get("{0}{1}".format(self.server_url, slug),
                            headers=self.auth_header,
-                           params=self.params)
+                           params=self.params,
+                           timeout=timeout)
         if req is None:
             raise APIError("Request is none!")
         return self.get_json(req)
@@ -78,6 +76,8 @@ class API:
         return self.make_request("exercises/{0}.json".format(id))
 
     def get_zip_stream(self, id):
+        if not self.tried_configuration:
+            self.db_configure()
         if not self.configured:
             raise APIError("API needs to be configured before use!")
         return requests.get("{0}exercises/{1}.zip".format(self.server_url, id),
@@ -86,6 +86,8 @@ class API:
                             params=self.params)
 
     def send_zip(self, id, file, params):
+        if not self.tried_configuration:
+            self.db_configure()
         if not self.configured:
             raise APIError("API needs to be configured before use!")
         return self.get_json(
