@@ -7,9 +7,11 @@ import subprocess
 import xml.etree.ElementTree as ET
 import time
 
-from tmc.errors import APIError, TMCError
+from tmc.errors import (APIError, TMCError, NotDownloaded, MissingProgram,
+                        WrongExerciseType)
 from tmc.spinner import SpinnerDecorator
 from tmc.models import Exercise
+from tmc.tests.basetest import test
 
 
 class Files:
@@ -17,7 +19,7 @@ class Files:
     def __init__(self, api):
         self.api = api
 
-    def download_file(self, id, force=False):
+    def download_file(self, id, force=False, update_java=False):
         exercise = Exercise.get(Exercise.tid == id)
         course = exercise.get_course()
         outpath = os.path.join(course.path)
@@ -29,9 +31,14 @@ class Files:
             print("Already downloaded, skipping.")
             exercise.is_downloaded = True
             exercise.save()
+            if update_java:
+                try:
+                    self.modify_java_target(exercise)
+                except TMCError:
+                    pass
             return
 
-        @SpinnerDecorator("Done!")
+        @SpinnerDecorator("Downloaded.", waitmsg="Downloading.")
         def inner(id):
             req = self.api.get_zip_stream(id)
             tmpfile = BytesIO()
@@ -45,102 +52,38 @@ class Files:
             exercise.save()
         inner(id)
 
-    def test_ant(self, path):
-        retcode = -1
-        out = None
-        try:
-            ret = subprocess.Popen(["ant", "clean", "test"],
-                                   stdout=subprocess.PIPE,
-                                   stderr=subprocess.PIPE,
-                                   cwd=path)
-            out = ret.communicate()[0].decode('utf-8')
-            retcode = ret.returncode
-        except OSError as e:
-            if e.errno is os.errno.ENOENT:
-                raise TMCError("You don't seem to have ant installed.")
-        if retcode != 0:
-            sys.stderr.write("\033[31m")
-            tests = glob(
-                os.path.join(path, "build", "test", "results", "*.xml"))
-            if len(tests) > 0:
-                for test in tests:
-                    tree = ET.parse(test)
-                    root = tree.getroot()
-                    for testcase in root.findall("testcase"):
-                        for failure in testcase.findall("failure"):
-                            sys.stderr.write(failure.text)
-            else:
-                for line in out.split("\n"):
-                    if "[javac] " in line:
-                        sys.stderr.write(line.split("[javac] ")[1] + "\n")
-            sys.stderr.write("\033[0m")
-            return False
-        return True
+        if update_java:
+            try:
+                self.modify_java_target(exercise)
+            except WrongExerciseType:
+                pass
 
-    def test_check(self, path):
-        out, err = "", ""
-        good = True
-        try:
-            ret = subprocess.Popen(["make", "clean", "all", "run-test"],
-                                   stdout=subprocess.PIPE,
-                                   stderr=subprocess.PIPE,
-                                   cwd=path)
-            out, err = ret.communicate()
-            out = out.decode("utf-8")
-            err = err.decode("utf-8")
-        except OSError as e:
-            if e.errno is os.errno.ENOENT:
-                raise TMCError("You don't have make installed")
-
-        testpath = os.path.join(path, "test", "tmc_test_results.xml")
-        if not os.path.isfile(testpath):
-            sys.stderr.write("\033[31m" + err + "\033[0m")
-            return False
-
-        sys.stderr.write("\033[31m")
-        ET.register_namespace("", "http://check.sourceforge.net/ns")
-        ns = "{http://check.sourceforge.net/ns}"
-        root = ET.parse(testpath).getroot()
-        for test in root.iter(ns + "test"):
-            if test.get("result") == "failure":
-                good = False
-                msg = test.find(ns + "message").text
-                func = test.find(ns + "fn").text
-                sys.stderr.write("{}: {}\n".format(func, msg))
-
-        sys.stderr.write("\033[0m")
-        return good
+    def modify_java_target(self, exercise, old="1.6", new="1.7"):
+        path = os.path.join(exercise.path(), "nbproject", "project.properties")
+        if not os.path.isfile(path):
+            raise WrongExerciseType("java")
+        lines = []
+        with open(path) as fp:
+            lines = fp.readlines()
+        for ind, line in enumerate(lines):
+            if line.startswith("javac") and line.endswith("=" + old + "\n"):
+                lines[ind] = line.replace(old, new)
+        with open(path, "w") as fp:
+            fp.write("".join(lines))
+        print("Changed Java target from {} to {}".format(old, new))
 
     def test(self, id):
-        exercise = Exercise.get(Exercise.tid == id)
-        outpath = exercise.path()
-        print("testing {0}".format(outpath))
-        if not os.path.isdir(outpath):
-            raise Exception("That exercise is not downloaded!")
-        exercise.is_downloaded = True
-        exercise.save()
-        # testing for what type of project this is
-        ret = None
-        if os.path.isfile(os.path.join(outpath, "build.xml")):
-            ret = self.test_ant(outpath)
-        elif os.path.isfile(os.path.join(outpath, "Makefile")):
-            ret = self.test_check(outpath)
-        if ret is None:
-            print("Unknown project type")
-        else:
-            if ret:
-                print("\033[32mOK!\033[0m")
-            else:
-                exit(-1)
+        test(Exercise.get(Exercise.tid == id))
 
     def submit(self, id, request_review=False, pastebin=False):
         exercise = Exercise.get(Exercise.tid == id)
         outpath = exercise.path()
         print("{0} -> {1}exercises/{2}.json".format(outpath,
-                                                    self.api.server_url, id))
+                                                    self.api.server_url,
+                                                    id))
         outpath = os.path.join(outpath, "src")
         if not os.path.isdir(outpath):
-            raise Exception("That exercise is not downloaded!")
+            raise NotDownloaded()
         exercise.is_downloaded = True
         exercise.save()
 
@@ -150,7 +93,8 @@ class Files:
         if pastebin:
             params["paste"] = "wolololo"
 
-        @SpinnerDecorator("Sent.")
+        @SpinnerDecorator("Submission has been sent.",
+                          waitmsg="Sending submission.")
         def inner():
             tmpfile = BytesIO()
             zipfp = zipfile.ZipFile(tmpfile, "w")
@@ -177,7 +121,8 @@ class Files:
             url = resp["submission_url"]
             submission_id = int(url.split(".json")[0].split("submissions/")[1])
 
-            @SpinnerDecorator("Results:")
+            @SpinnerDecorator("Results:",
+                              "Waiting for results.")
             def inner():
                 while True:
                     try:
@@ -198,7 +143,8 @@ class Files:
                         sys.stderr.write("{0}:\n  {1}\n".format(
                             testcase["name"], testcase["message"]))
                 sys.stderr.write("".join(["\033[33mFor better details run `tmc"
-                                          " test " + str(id) + "`\033[0m\n"]))
+                                          " test --id ", str(id),
+                                          "`\033[0m\n"]))
                 exit(-1)
             elif data["status"] == "ok":
                 print("\033[32mPoints [{0}]\033[0m".format(
