@@ -3,9 +3,9 @@ from peewee import (Model, CharField, BooleanField, DateField, IntegerField,
                     ForeignKeyField, SqliteDatabase)
 from tmc.errors import NoCourseSelected, NoExerciseSelected
 
-sqlite = SqliteDatabase(os.path.join(os.path.expanduser("~"),
-                                     '.config',
-                                     'tmc.db'))
+sqlite = SqliteDatabase(
+    os.path.join(os.path.expanduser("~"), ".config", "tmc.db")
+)
 sqlite.connect()
 
 
@@ -16,7 +16,7 @@ class BaseModel(Model):
 
 
 class Course(BaseModel):
-    tid = IntegerField()
+    tid = IntegerField(unique=True)
     name = CharField()
     is_selected = BooleanField(default=False)
     path = CharField(default="")
@@ -37,9 +37,15 @@ class Course(BaseModel):
     def __str__(self):
         return "Course \"{}\" (ID {})".format(self.name, self.tid)
 
+    def __repr__(self):
+        return str(self)
+
+    def menuname(self):
+        return self.name
+
 
 class Exercise(BaseModel):
-    tid = IntegerField()
+    tid = IntegerField(unique=True)
     name = CharField()
     course = ForeignKeyField(Course, related_name="exercises")
     is_selected = BooleanField(default=False)
@@ -61,6 +67,10 @@ class Exercise(BaseModel):
         self.is_selected = True
         self.save()
 
+    def update_downloaded(self):
+        self.is_downloaded = os.path.isdir(self.path())
+        self.save()
+
     @staticmethod
     def get_selected():
         is_selected = Exercise.select().where(Exercise.is_selected == True)
@@ -75,6 +85,30 @@ class Exercise(BaseModel):
     def __str__(self):
         return "Exercise \"{}\" (ID {})".format(self.name, self.tid)
 
+    def __repr__(self):
+        return str(self)
+
+    def menuname(self):
+        short, rest = "", ""
+        if "-" in self.name:
+            rest = self.name.split("-")[-1]
+        else:
+            rest = self.name
+        if "." in rest:
+            split = rest.split(".")
+            short = split[-1]
+            rest = ".".join(split[0:-1])
+        realname = ""
+        for c in short:
+            if c.isupper():
+                if len(realname) == 0:
+                    realname += c
+                else:
+                    realname += " " + c
+            else:
+                realname += c
+        return rest.replace("_", " - ") + " - " + realname
+
 
 class Config(BaseModel):
     name = CharField(primary_key=True)
