@@ -6,8 +6,10 @@ def yn_prompt(msg, default=True):
 
 
 def custom_prompt(msg, options, default):
-    sure = input("{0} [{1}]: ".format(msg,
-                                      "/".join([x.upper() if x == default else x.lower() for x in options])))
+    formatted_options = [
+        x.upper() if x == default else x.lower() for x in options
+    ]
+    sure = input("{0} [{1}]: ".format(msg, "/".join(formatted_options)))
     if len(sure) == 0:
         return default
     for option in options:
