@@ -10,10 +10,10 @@ class Spinner(threading.Thread):
     A spinny spinner.
     """
 
-    def __init__(self, msg=""):
+    def __init__(self, msg="", waitmsg="Please wait"):
         threading.Thread.__init__(self)
         self.chars = ['\\', '|', '/', '-']
-        self.template = "({0}) Please wait."
+        self.template = "({0}) " + waitmsg
         self.msg = msg
         self.index = 0
         self.spinning = True
@@ -35,13 +35,20 @@ class Spinner(threading.Thread):
         self.index = 0
 
 
-def SpinnerDecorator(msg=""):
+def SpinnerDecorator(msg="", waitmsg="Please wait"):
     def Decorator(func):
         @functools.wraps(func)
         def wrapper(*args, **kwargs):
-            spin = Spinner(msg)
+            spin = Spinner(msg=msg, waitmsg=waitmsg)
             spin.start()
-            a = func(*args, **kwargs)
+            a = None
+            try:
+                a = func(*args, **kwargs)
+            except Exception as e:
+                spin.msg = "Something went wrong: "
+                spin.stopspin()
+                spin.join()
+                raise e
             spin.stopspin()
             spin.join()
             return a
